#~ '(C) Links System Software 2009
#~ 'Feedback information
#~ 'www: http://lss2008.livejournal.com/
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru
#~ 'icq: 328-958-369

#~ Little plug-in, which creates approximate 3d mesh from a set of selected construction points.
#~ How to use:
#~ 1)	put pnts2mesh.rb to Google SketchUp 6\Plugins folder
#~ 2)	start SketchUp 6
#~ 3)	draw some construction points (or import dxf file with points cloud)
#~ 4)	select a set of construction points, which you want to be processed, on the screen
#~ 5)	choose Plugins>> Create 3d mesh from construction points... command from the main menu
#~ 6)	Mesh Settings input box appears, specify appropriate values for Cells count and click Ok (note: big values of Cells count takes a lot of time to process)

# method to calculate z of the mesh vertex
def z_hyp_r_power(selection,meshpt)
  power=3
  hyprsum=0
  zaverage=0
  ptcnt=0
  rn=0
  deltaz=0
  deltazeval=0
  selection.each {|constpt|
    pt=constpt.position
    rn=(Math.sqrt((meshpt.x - pt.x) ** 2 + (meshpt.y - pt.y) ** 2)) ** power
    
     if rn==0
       z=pt.z
       meshpt.z=z
       puts "#{z}"
       return
      end
     hyprsum = hyprsum + 1/rn
     zaverage = zaverage + pt.z
     ptcnt += 1
  }
  zaverage=zaverage/ptcnt

  selection.each {|constpt|
    pt=constpt.position
    rn=(Math.sqrt((meshpt.x - pt.x) ** 2 + (meshpt.y - pt.y) ** 2)) ** power
    deltaz=pt.z - zaverage
    deltazeval = deltazeval + deltaz * ((hyprsum - 1 / rn) / hyprsum)
  }
  z = zaverage-deltazeval
  meshpt.z = z
  end

# method of 3d mesh creation
def make_relief_mesh(m, n)
  model=Sketchup.active_model
  selection=model.selection
  if selection.count==0
    UI.messagebox "No construction points selected" 
    return
  end
  
  bbox=Geom::BoundingBox.new
  selection.each {|ent| bbox.add ent.position}
  pt1=bbox.corner 0
  pt2=bbox.corner 1
  pt3=bbox.corner 2
  pt4=bbox.corner 3
  msize=m
  nsize=n
  
  mesh = Geom::PolygonMesh.new 
  model=Sketchup.active_model
  entities = model.entities 
  
  i=0
  while i<msize
    j=0
    while j<nsize
      point1=Geom::Point3d.new pt1+[i*(pt2.x-pt1.x)/msize, j*(pt3.y-pt1.y)/nsize,0]
      z_hyp_r_power selection,point1
      point2=Geom::Point3d.new pt1+[(i+1)*(pt2.x-pt1.x)/msize, j*(pt3.y-pt1.y)/nsize,0]
      z_hyp_r_power selection,point2
      point3=Geom::Point3d.new pt1+[i*(pt2.x-pt1.x)/msize, (j+1)*(pt3.y-pt1.y)/nsize,0]
      z_hyp_r_power selection,point3
      point4=Geom::Point3d.new pt1+[(i+1)*(pt2.x-pt1.x)/msize, (j+1)*(pt3.y-pt1.y)/nsize,0]
      z_hyp_r_power selection,point4
      
      mesh.add_point point1
      mesh.add_point point2
      mesh.add_point point3
      mesh.add_point point4
      
      mesh.add_polygon point1, point2, point3
      mesh.add_polygon point2, point4, point3
      
      j+=1
    end
    i+=1
  end
  group = Sketchup.active_model.entities.add_group
  group.entities.add_faces_from_mesh mesh, 12
end


tool_menu = UI.menu "Plugins" 
 tool_menu.add_item("Create 3d mesh from construction points...") {
  prompts = ["Cells along X axis count: ", "Cells along Y axis count: "]
  defaults= [10, 10]
  mn=UI.inputbox prompts, defaults, "Mesh settings"
  make_relief_mesh mn[0],mn[1]
}